from netbox.api.routers import NetBoxRouter
from . import views


router = NetBoxRouter()
router.register('ldp', views.LDP_ViewSet)
router.register('bgp-ce', views.BGP_CE_ViewSet, basename='bgp_ce')
router.register('bgp-pe', views.BGP_PE_ViewSet, basename='bgp_pe')
router.register('bgp-mesh', views.BGP_Mesh_ViewSet, basename='bgp_mesh')
router.register('address-family', views.Address_Family_ViewSet, basename='address_family')
router.register('mpls-instance', views.MPLS_Instance_ViewSet, basename='mpls_instance')

urlpatterns = router.urls



from typing import Dict

from dcim.models.device_components import Interface
from ipam.models.ip import IPAddress
from django.db.models import Q
from utilities.forms.fields import CommentField
from utilities.forms.fields import DynamicModelMultipleChoiceField
from utilities.forms.fields.dynamic import DynamicModelMultipleChoiceField, DynamicModelChoiceField

from netbox.forms import NetBoxModelForm

from . import models

# HTML rendering


class LDP_Form(NetBoxModelForm):
    """Form to fill out LDP instance."""

    comments = CommentField()

    ldp_enabled_interfaces = DynamicModelMultipleChoiceField(
        queryset=Interface.objects.all(),
        required=False,
        label="LDP enabled interfaces",
        query_params={
            "device_id": "$device",
        },
    )

    class Meta:
        model = models.LDP
        fields = ('device', 'auto_range', 'ldp_enabled_interfaces', 'label_start', 'label_end', 'comments')

    def clean(self):
        data: Dict = super().clean()
        start = data.get('label_start')
        end = data.get('label_end')
        auto = data.get('auto_range')
        if not auto:
            if start is not None and end is not None:
                min_value, max_value = 0, 1048575
                if not (min_value <= int(start) <= max_value):
                    self.add_error('label_start', f"Label outside range: [{min_value};{max_value}]")
                if not (min_value <= int(end) <= max_value):
                    self.add_error('label_end', f"Label outside range: [{min_value};{max_value}]")
                if not (int(end) > int(start)):
                    msg = "Label end must exceed label start."
                    self.add_error('label_start', msg)
                    self.add_error('label_end', msg)
            else:
                if start is None:
                    self.add_error('label_start', f"Missing value.")
                if end is None:
                    self.add_error('label_end', f"Missing value.")
        return data


class BGP_CE_Form(NetBoxModelForm):
    """Form to fill out BGP CE instance."""

    comments = CommentField()

    class Meta:
        model = models.BGP_CE
        fields = ('name', 'ip_addresses', 'asn', 'comments')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['ip_addresses'].choices = [
            (ip_address.pk, f"{ip_address.address}" + (f" - {ip_address.vrf}" if ip_address.vrf is not None else ""))
            for ip_address in IPAddress.objects.all()
        ]


class BGP_PE_Form(NetBoxModelForm):
    """Form to fill out BGP PE instance."""

    comments = CommentField()

    update_source_interface = DynamicModelChoiceField(
        queryset=Interface.objects.all(),
        required=True,
        label="Interface for BGP exchange",
        query_params={
            "device_id": "$device",
        },
    )

    class Meta:
        model = models.BGP_PE
        fields = (
            'device',
            'update_source_interface',
            'asn',
            'advertised_customers',
            'comments',
        )


class BGP_Mesh_Form(NetBoxModelForm):
    """Form to fill out BGP Mesh Neighbourhood instance."""

    comments = CommentField()

    bgp_mesh = DynamicModelMultipleChoiceField(
        queryset=models.BGP_PE.objects.all(),
        label="BGP PEs",
        help_text="Select BGP PE devices which are part of this mesh.",
        required=True,
    )

    class Meta:
        model = models.BGP_Mesh
        fields = (
            'name',
            'bgp_mesh',
            'address_family',
            'comments',
        )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        mesh: models.BGP_Mesh = kwargs['instance']
        queryset = models.BGP_PE.objects.filter(Q(bgp_mesh=None) | Q(bgp_mesh=mesh.pk))
        self.fields['bgp_mesh'].queryset = queryset


class Address_Family_Form(NetBoxModelForm):
    """Form to fill out Address Family instance."""

    comments = CommentField()

    class Meta:
        model = models.Address_Family
        fields = (
            'name',
            'vpnv4_address_family',
            'vpnv4_community',
            'vpnv6_address_family',
            'vpnv6_community',
            'comments',
        )


class MPLS_Instance_Form(NetBoxModelForm):
    """Form to fill out Address Family instance."""

    comments = CommentField()

    class Meta:
        model = models.MPLS_Instance
        fields = (
            'name',
            'ibgp',
            'service',
            'comments',
        )



from netbox.api.viewsets import NetBoxModelViewSet

from .. import models
from . import serializers


class LDP_ViewSet(NetBoxModelViewSet):
    queryset = models.LDP.objects.all()
    serializer_class = serializers.LDPSerializer


class BGP_CE_ViewSet(NetBoxModelViewSet):
    queryset = models.BGP_CE.objects.all()
    serializer_class = serializers.BGP_CESerializer


class BGP_PE_ViewSet(NetBoxModelViewSet):
    queryset = models.BGP_PE.objects.all()
    serializer_class = serializers.BGP_PESerializer


class BGP_Mesh_ViewSet(NetBoxModelViewSet):
    queryset = models.BGP_Mesh.objects.all()
    serializer_class = serializers.BGP_MeshSerializer


class Address_Family_ViewSet(NetBoxModelViewSet):
    queryset = models.Address_Family.objects.all()
    serializer_class = serializers.Address_FamilySerializer


class MPLS_Instance_ViewSet(NetBoxModelViewSet):
    queryset = models.MPLS_Instance.objects.all()
    serializer_class = serializers.MPLS_InstanceSerializer



from graphene import ObjectType, Field, ResolveInfo
from netbox.graphql.types import NetBoxObjectType
from netbox.graphql.fields import ObjectField, ObjectListField
from . import models


#
# Object types
#


class LDP_Type(NetBoxObjectType):
    class Meta:
        model = models.LDP
        fields = '__all__'


class BGP_CE_Type(NetBoxObjectType):
    class Meta:
        model = models.BGP_CE
        fields = '__all__'


class BGP_PE_Type(NetBoxObjectType):
    class Meta:
        model = models.BGP_PE
        fields = '__all__'

    bgp_mesh = Field(lambda: BGP_Mesh_Type)

    def resolve_bgp_mesh(root: 'BGP_PE_Type', info: ResolveInfo):
        bgp_meshes = root.bgp_mesh.all()
        if bgp_meshes:
            return bgp_meshes.first()
        return None


class BGP_Mesh_Type(NetBoxObjectType):
    class Meta:
        model = models.BGP_Mesh
        fields = '__all__'


class Address_Family_Type(NetBoxObjectType):
    class Meta:
        model = models.Address_Family
        fields = '__all__'


class MPLS_Instance_Type(NetBoxObjectType):
    class Meta:
        model = models.MPLS_Instance
        fields = '__all__'


#
# Queries
#


class Query(ObjectType):
    ldp = ObjectField(LDP_Type)
    ldp_list = ObjectListField(LDP_Type)

    bgp_ce = ObjectField(BGP_CE_Type)
    bgp_ce_list = ObjectListField(BGP_CE_Type)

    bgp_pe = ObjectField(BGP_PE_Type)
    bgp_pe_list = ObjectListField(BGP_PE_Type)

    bgp_mesh = ObjectField(BGP_Mesh_Type)
    bgp_mesh_list = ObjectListField(BGP_Mesh_Type)

    address_family = ObjectField(Address_Family_Type)
    address_family_list = ObjectListField(Address_Family_Type)

    mpls_instance = ObjectField(MPLS_Instance_Type)
    mpls_instance_list = ObjectListField(MPLS_Instance_Type)


schema = Query



# from netbox.filtersets import NetBoxModelFilterSet
# from .models import AccessListRule


# class AccessListRuleFilterSet(NetBoxModelFilterSet):

#     class Meta:
#         model = AccessListRule
#         fields = ('id', 'access_list', 'index', 'protocol', 'action')

#     def search(self, queryset, name, value):
#         return queryset.filter(description__icontains=value)



from extras.plugins import PluginConfig
from .version import __version__


class ArgosPluginConfig(PluginConfig):
    name = 'argos_netbox'
    verbose_name = 'Argos'
    description = 'Manage simple MPLS in NetBox'
    version = __version__
    base_url = 'argos'
    min_version = '3.4.0'


config = ArgosPluginConfig



import django_tables2 as tables
from django.urls import reverse
from django.utils.html import format_html

from netbox.tables import NetBoxTable, ChoiceFieldColumn
from . import models, config

# List of entries


class LDP_Table(NetBoxTable):
    device = tables.Column(
        linkify=True,
    )

    range = tables.Column()

    class Meta(NetBoxTable.Meta):
        model = models.LDP
        fields = ('id', 'device', 'range', 'ldp_enabled_interfaces', 'last_updated')
        default_columns = ('id', 'device', 'range', 'ldp_enabled_interfaces')

    def render_ldp_enabled_interfaces(self, record):
        interfaces = record.ldp_enabled_interfaces.all()
        interface_links = []
        for interface in interfaces:
            url = reverse('dcim:interface', kwargs={'pk': interface.pk})
            interface_links.append(format_html('<a href="{}">{}</a>', url, interface))
        return format_html(', '.join(interface_links))


class BGP_CE_Table(NetBoxTable):
    asn = tables.Column(
        linkify=True,
    )

    class Meta(NetBoxTable.Meta):
        model = models.BGP_CE
        fields = ('id', 'name', 'asn', 'last_updated')
        default_columns = ('id', 'name', 'asn')


class BGP_PE_Table(NetBoxTable):
    device = tables.Column(
        linkify=True,
    )
    update_source_interface = tables.Column(
        linkify=True,
    )
    asn = tables.Column(
        linkify=True,
    )

    class Meta(NetBoxTable.Meta):
        model = models.BGP_PE
        fields = ('id', 'device', 'update_source_interface', 'asn', 'advertised_customers', 'last_updated')
        default_columns = ('id', 'device', 'update_source_interface', 'asn', 'advertised_customers')

    def render_advertised_customers(self, record):
        customers = record.advertised_customers.all()
        customer_links = []
        for customer in customers:
            url = reverse(f'plugins:{config.name}:bgp_ce', kwargs={'pk': customer.pk})
            customer_links.append(format_html('<a href="{}">{}</a>', url, customer))
        return format_html(', '.join(customer_links))


class BGP_Mesh_Table(NetBoxTable):
    bgp_mesh = tables.ManyToManyColumn(
        linkify=True,
    )

    name = tables.Column(
        linkify=True,
    )

    address_family = tables.Column(
        linkify=True,
    )

    class Meta(NetBoxTable.Meta):
        model = models.BGP_Mesh
        fields = ('id', 'bgp_mesh', 'name', 'address_family' 'last_updated')
        default_columns = ('id', 'bgp_mesh', 'name', 'address_family')


class Address_Family_Table(NetBoxTable):
    name = tables.Column(
        linkify=True,
    )

    vpnv4_community = ChoiceFieldColumn()

    vpnv6_community = ChoiceFieldColumn()

    class Meta(NetBoxTable.Meta):
        model = models.Address_Family
        fields = ('id', 'name', 'vpnv4_community', 'vpnv6_community', 'last_updated')
        default_columns = ('id', 'name', 'vpnv4_community', 'vpnv6_community')


class MPLS_Instance_Table(NetBoxTable):
    name = tables.Column(
        linkify=True,
    )

    ibgp = tables.Column(
        linkify=True,
    )

    service = ChoiceFieldColumn()

    class Meta(NetBoxTable.Meta):
        model = models.MPLS_Instance
        fields = ('id', 'name', 'ibgp', 'service', 'last_updated')
        default_columns = ('id', 'name', 'ibgp', 'service')



from netbox.views import generic
from . import forms, models, tables


#
# Request handlers, response rendering
#


# LDP
class LDP_View(generic.ObjectView):
    queryset = models.LDP.objects.all()


class LDP_ListView(generic.ObjectListView):
    queryset = models.LDP.objects.all()
    table = tables.LDP_Table


class LDP_EditView(generic.ObjectEditView):
    queryset = models.LDP.objects.all()
    form = forms.LDP_Form
    template_name = 'argos_netbox/ldp_edit.html'


class LDP_DeleteView(generic.ObjectDeleteView):
    queryset = models.LDP.objects.all()


# BGP CE
class BGP_CE_View(generic.ObjectView):
    queryset = models.BGP_CE.objects.all()


class BGP_CE_ListView(generic.ObjectListView):
    queryset = models.BGP_CE.objects.all()
    table = tables.BGP_CE_Table


class BGP_CE_EditView(generic.ObjectEditView):
    queryset = models.BGP_CE.objects.all()
    form = forms.BGP_CE_Form


class BGP_CE_DeleteView(generic.ObjectDeleteView):
    queryset = models.BGP_CE.objects.all()


# BGP PE
class BGP_PE_View(generic.ObjectView):
    queryset = models.BGP_PE.objects.all()


class BGP_PE_ListView(generic.ObjectListView):
    queryset = models.BGP_PE.objects.all()
    table = tables.BGP_PE_Table


class BGP_PE_EditView(generic.ObjectEditView):
    queryset = models.BGP_PE.objects.all()
    form = forms.BGP_PE_Form


class BGP_PE_DeleteView(generic.ObjectDeleteView):
    queryset = models.BGP_PE.objects.all()


# BGP Mesh
class BGP_Mesh_View(generic.ObjectView):
    queryset = models.BGP_Mesh.objects.all()


class BGP_Mesh_ListView(generic.ObjectListView):
    queryset = models.BGP_Mesh.objects.all()
    table = tables.BGP_Mesh_Table


class BGP_Mesh_EditView(generic.ObjectEditView):
    queryset = models.BGP_Mesh.objects.all()
    form = forms.BGP_Mesh_Form


class BGP_Mesh_DeleteView(generic.ObjectDeleteView):
    queryset = models.BGP_Mesh.objects.all()


# Address Family
class Address_Family_View(generic.ObjectView):
    queryset = models.Address_Family.objects.all()


class Address_Family_ListView(generic.ObjectListView):
    queryset = models.Address_Family.objects.all()
    table = tables.Address_Family_Table


class Address_Family_EditView(generic.ObjectEditView):
    queryset = models.Address_Family.objects.all()
    form = forms.Address_Family_Form
    template_name = 'argos_netbox/address_family_edit.html'


class Address_Family_DeleteView(generic.ObjectDeleteView):
    queryset = models.Address_Family.objects.all()


# MPLS Instance
class MPLS_Instance_View(generic.ObjectView):
    queryset = models.MPLS_Instance.objects.all()


class MPLS_Instance_ListView(generic.ObjectListView):
    queryset = models.MPLS_Instance.objects.all()
    table = tables.MPLS_Instance_Table


class MPLS_Instance_EditView(generic.ObjectEditView):
    queryset = models.MPLS_Instance.objects.all()
    form = forms.MPLS_Instance_Form


class MPLS_Instance_DeleteView(generic.ObjectDeleteView):
    queryset = models.MPLS_Instance.objects.all()



from extras.api.serializers import CustomFieldSerializer
from netbox.api.serializers.base import ValidatedModelSerializer
from rest_framework import serializers

from netbox.api.serializers import WritableNestedSerializer
from .. import models

from .. import config


class BaseModelSerializer(CustomFieldSerializer, ValidatedModelSerializer):
    """
    Adds support for custom fields and tags.
    """

    pass


class NestedLDPSerializer(WritableNestedSerializer):
    url = serializers.HyperlinkedIdentityField(view_name=f'plugins:{config.name}:ldp')

    class Meta:
        model = models.LDP
        fields = (
            'id',
            'url',
            'device',
            'label_start',
            'label_end',
            'auto_range',
            'last_updated',
        )


class LDPSerializer(BaseModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name=f'plugins:{config.name}:ldp')

    class Meta:
        model = models.LDP
        fields = (
            'id',
            'display',
            'custom_fields',  # From BaseModelSerializer
            'created',
            'last_updated',  # From NetBoxModel
            'url',  # From RouteTargetSerializer
            'device',
            'label_start',
            'label_end',
            'auto_range',
            'comments',  # From LDP
        )


class Nested_BGP_CESerializer(WritableNestedSerializer):
    url = serializers.HyperlinkedIdentityField(view_name=f'plugins:{config.name}:bgp_ce')

    class Meta:
        model = models.BGP_CE
        fields = (
            'id',
            'url',
            'name',
            'ip_addresses',
            'asn',
            'comments',
            'last_updated',
        )


class BGP_CESerializer(BaseModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name=f'plugins:{config.name}:bgp_ce')

    class Meta:
        model = models.BGP_CE
        fields = (
            'id',
            'display',
            'custom_fields',  # From BaseModelSerializer
            'created',
            'last_updated',  # From NetBoxModel
            'url',  # From RouteTargetSerializer
            'name',  # From BGP_CE
            'ip_addresses',
            'asn',
            'comments',
        )


class Nested_BGP_PESerializer(WritableNestedSerializer):
    url = serializers.HyperlinkedIdentityField(view_name=f'plugins:{config.name}:bgp_pe')

    class Meta:
        model = models.BGP_PE
        fields = (
            'id',
            'url',
            'device',
            'update_source_interface',
            'asn',
            'advertised_customers',
            'comments',
            'last_updated',
        )


class BGP_PESerializer(BaseModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name=f'plugins:{config.name}:bgp_pe')

    class Meta:
        model = models.BGP_PE
        fields = (
            'id',
            'display',
            'custom_fields',  # From BaseModelSerializer
            'created',
            'last_updated',  # From NetBoxModel
            'url',  # From RouteTargetSerializer
            'device',  # From BGP_PE
            'update_source_interface',
            'asn',
            'advertised_customers',
            'comments',
        )


class Nested_BGP_MeshSerializer(WritableNestedSerializer):
    url = serializers.HyperlinkedIdentityField(view_name=f'plugins:{config.name}:bgp_mesh')

    class Meta:
        model = models.BGP_Mesh
        fields = (
            'id',
            'url',
            'name',
            'bgp_mesh',
            'comments',
            'last_updated',
        )


class BGP_MeshSerializer(BaseModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name=f'plugins:{config.name}:bgp_mesh')

    class Meta:
        model = models.BGP_Mesh
        fields = (
            'id',
            'display',
            'custom_fields',  # From BaseModelSerializer
            'created',
            'last_updated',  # From NetBoxModel
            'url',  # From RouteTargetSerializer
            'bgp_mesh',  # From BGP_Mesh
            'name',
            'comments',
        )


class Nested_Address_Family_Serializer(WritableNestedSerializer):
    url = serializers.HyperlinkedIdentityField(view_name=f'plugins:{config.name}:address_family')

    class Meta:
        model = models.Address_Family
        fields = (
            'id',
            'url',
            'name',
            'vpnv4_address_family',
            'vpnv4_community',
            'vpnv6_address_family',
            'vpnv6_community',
            'comments',
            'last_updated',
        )


class Address_FamilySerializer(BaseModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name=f'plugins:{config.name}:address_family')

    class Meta:
        model = models.Address_Family
        fields = (
            'id',
            'display',
            'custom_fields',  # From BaseModelSerializer
            'created',
            'last_updated',  # From NetBoxModel
            'url',  # From RouteTargetSerializer
            'name',  # From BGP_Mesh
            'vpnv4_address_family',
            'vpnv4_community',
            'vpnv6_address_family',
            'vpnv6_community',
            'comments',
        )


class Nested_MPLS_Instance_Serializer(WritableNestedSerializer):
    url = serializers.HyperlinkedIdentityField(view_name=f'plugins:{config.name}:mpls_instance')

    class Meta:
        model = models.MPLS_Instance
        fields = (
            'id',
            'url',
            'ibgp',
            'service',
            'name',
            'comments',
            'last_updated',
        )


class MPLS_InstanceSerializer(BaseModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name=f'plugins:{config.name}:mpls_instance')

    class Meta:
        model = models.MPLS_Instance
        fields = (
            'id',
            'display',
            'custom_fields',  # From BaseModelSerializer
            'created',
            'last_updated',  # From NetBoxModel
            'url',  # From RouteTargetSerializer
            'ibgp',  # From BGP_Mesh
            'service',
            'name',
            'comments',
        )



from django.urls import path

from netbox.views.generic import ObjectChangeLogView
from . import models, views


urlpatterns = (
    # LDP
    path('ldp/', views.LDP_ListView.as_view(), name='ldp_list'),
    path('ldp/add/', views.LDP_EditView.as_view(), name='ldp_add'),
    path('ldp/<int:pk>/', views.LDP_View.as_view(), name='ldp'),
    path('ldp/<int:pk>/edit/', views.LDP_EditView.as_view(), name='ldp_edit'),
    path('ldp/<int:pk>/delete/', views.LDP_DeleteView.as_view(), name='ldp_delete'),
    path(
        'ldp/<int:pk>/changelog/',
        ObjectChangeLogView.as_view(),
        name='ldp_changelog',
        kwargs={'model': models.LDP},
    ),
    # BGP CE
    path('bgp-ce/', views.BGP_CE_ListView.as_view(), name='bgp_ce_list'),
    path('bgp-ce/add/', views.BGP_CE_EditView.as_view(), name='bgp_ce_add'),
    path('bgp-ce/<int:pk>/', views.BGP_CE_View.as_view(), name='bgp_ce'),
    path('bgp-ce/<int:pk>/edit/', views.BGP_CE_EditView.as_view(), name='bgp_ce_edit'),
    path('bgp-ce/<int:pk>/delete/', views.BGP_CE_DeleteView.as_view(), name='bgp_ce_delete'),
    path(
        'bgp-ce/<int:pk>/changelog/',
        ObjectChangeLogView.as_view(),
        name='bgp_ce_changelog',
        kwargs={'model': models.BGP_CE},
    ),
    # BGP PE
    path('bgp-pe/', views.BGP_PE_ListView.as_view(), name='bgp_pe_list'),
    path('bgp-pe/add/', views.BGP_PE_EditView.as_view(), name='bgp_pe_add'),
    path('bgp-pe/<int:pk>/', views.BGP_PE_View.as_view(), name='bgp_pe'),
    path('bgp-pe/<int:pk>/edit/', views.BGP_PE_EditView.as_view(), name='bgp_pe_edit'),
    path('bgp-pe/<int:pk>/delete/', views.BGP_PE_DeleteView.as_view(), name='bgp_pe_delete'),
    path(
        'bgp-pe/<int:pk>/changelog/',
        ObjectChangeLogView.as_view(),
        name='bgp_pe_changelog',
        kwargs={'model': models.BGP_PE},
    ),
    # BGP Mesh
    path('bgp-mesh/', views.BGP_Mesh_ListView.as_view(), name='bgp_mesh_list'),
    path('bgp-mesh/add/', views.BGP_Mesh_EditView.as_view(), name='bgp_mesh_add'),
    path('bgp-mesh/<int:pk>/', views.BGP_Mesh_View.as_view(), name='bgp_mesh'),
    path('bgp-mesh/<int:pk>/edit/', views.BGP_Mesh_EditView.as_view(), name='bgp_mesh_edit'),
    path('bgp-mesh/<int:pk>/delete/', views.BGP_Mesh_DeleteView.as_view(), name='bgp_mesh_delete'),
    path(
        'bgp-mesh/<int:pk>/changelog/',
        ObjectChangeLogView.as_view(),
        name='bgp_mesh_changelog',
        kwargs={'model': models.BGP_Mesh},
    ),
    # Address Family
    path('address-family/', views.Address_Family_ListView.as_view(), name='address_family_list'),
    path('address-family/add/', views.Address_Family_EditView.as_view(), name='address_family_add'),
    path('address-family/<int:pk>/', views.Address_Family_View.as_view(), name='address_family'),
    path('address-family/<int:pk>/edit/', views.Address_Family_EditView.as_view(), name='address_family_edit'),
    path('address-family/<int:pk>/delete/', views.Address_Family_DeleteView.as_view(), name='address_family_delete'),
    path(
        'address-family/<int:pk>/changelog/',
        ObjectChangeLogView.as_view(),
        name='address_family_changelog',
        kwargs={'model': models.Address_Family},
    ),
    # MPLS Instance
    path('mpls-instance/', views.MPLS_Instance_ListView.as_view(), name='mpls_instance_list'),
    path('mpls-instance/add/', views.MPLS_Instance_EditView.as_view(), name='mpls_instance_add'),
    path('mpls-instance/<int:pk>/', views.MPLS_Instance_View.as_view(), name='mpls_instance'),
    path('mpls-instance/<int:pk>/edit/', views.MPLS_Instance_EditView.as_view(), name='mpls_instance_edit'),
    path('mpls-instance/<int:pk>/delete/', views.MPLS_Instance_DeleteView.as_view(), name='mpls_instance_delete'),
    path(
        'mpls-instance/<int:pk>/changelog/',
        ObjectChangeLogView.as_view(),
        name='mpls_instance_changelog',
        kwargs={'model': models.MPLS_Instance},
    ),
)



# Generated by Django 4.1.8 on 2023-05-07 06:52

import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import taggit.managers
import utilities.json


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ('dcim', '0167_module_status'),
        ('extras', '0084_staging'),
    ]

    operations = [
        migrations.CreateModel(
            name='LDP',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder),
                ),
                ('auto_range', models.BooleanField(default=True)),
                (
                    'label_start',
                    models.PositiveIntegerField(
                        blank=True,
                        null=True,
                        validators=[
                            django.core.validators.MinValueValidator(0),
                            django.core.validators.MaxValueValidator(1048575),
                        ],
                    ),
                ),
                (
                    'label_end',
                    models.PositiveIntegerField(
                        blank=True,
                        null=True,
                        validators=[
                            django.core.validators.MinValueValidator(0),
                            django.core.validators.MaxValueValidator(1048575),
                        ],
                    ),
                ),
                ('comments', models.TextField()),
                ('device', models.OneToOneField(on_delete=django.db.models.deletion.PROTECT, to='dcim.device')),
                (
                    'ldp_enabled_interfaces',
                    models.ManyToManyField(
                        limit_choices_to=models.Q(('device_id', models.F('device'))), to='dcim.interface'
                    ),
                ),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'verbose_name': 'LDP Participant',
                'verbose_name_plural': 'LDP Participants',
            },
        ),
        migrations.CreateModel(
            name='BGP_CE',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder),
                ),
                ('name', models.CharField(max_length=50)),
                ('comments', models.TextField()),
                ('asn', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='ipam.asn')),
                ('ip_addresses', models.ManyToManyField(to='ipam.ipaddress')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'verbose_name': 'BGP Customer Edge',
                'verbose_name_plural': 'BGP Customer Edges',
            },
        ),
        migrations.CreateModel(
            name='BGP_PE',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder),
                ),
                ('comments', models.TextField()),
                ('advertised_customers', models.ManyToManyField(blank=True, to='argos_netbox.bgp_ce')),
                ('asn', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='ipam.asn')),
                ('device', models.OneToOneField(on_delete=django.db.models.deletion.PROTECT, to='dcim.device')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
                (
                    'update_source_interface',
                    models.OneToOneField(
                        limit_choices_to=models.Q(('device_id', models.F('device'))),
                        on_delete=django.db.models.deletion.PROTECT,
                        to='dcim.interface',
                    ),
                ),
            ],
            options={
                'verbose_name': 'BGP Provider Edge',
                'verbose_name_plural': 'BGP Provider Edges',
            },
        ),
        migrations.CreateModel(
            name='Address_Family',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder),
                ),
                ('comments', models.TextField()),
                ('vpnv4_address_family', models.BooleanField(default=True)),
                ('vpnv4_community', models.CharField(max_length=50, null=True, blank=True)),
                ('vpnv6_address_family', models.BooleanField(default=True)),
                ('vpnv6_community', models.CharField(max_length=50, null=True, blank=True)),
                ('name', models.CharField(max_length=50)),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'verbose_name': 'Address Family',
                'verbose_name_plural': 'Address Families',
            },
        ),
        migrations.CreateModel(
            name='BGP_Mesh',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder),
                ),
                ('comments', models.TextField()),
                ('bgp_mesh', models.ManyToManyField(blank=False, related_name='bgp_mesh', to='argos_netbox.bgp_pe')),
                (
                    'address_family',
                    models.ForeignKey(
                        blank=False, on_delete=django.db.models.deletion.PROTECT, to='argos_netbox.address_family'
                    ),
                ),
                ('name', models.CharField(max_length=50)),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'verbose_name': 'BGP Mesh',
                'verbose_name_plural': 'BGP Meshes',
            },
        ),
        migrations.CreateModel(
            name='MPLS_Instance',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder),
                ),
                ('service', models.CharField(blank=False, max_length=50, null=False)),
                ('name', models.CharField(max_length=50)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('comments', models.TextField()),
                (
                    'ibgp',
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.PROTECT, related_name='ibgp', to='argos_netbox.bgp_mesh'
                    ),
                ),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'verbose_name': 'MPLS Instance',
                'verbose_name_plural': 'MPLS Instances',
            },
        ),
    ]



from django import template

register = template.Library()


@register.inclusion_tag('argos_netbox/util/hide_if_checked.html')
def hide_if_checked(checkbox_field_name, block_id, reverse=False):
    reverse = 1 if reverse else 0
    return {'checkbox_field_name': checkbox_field_name, 'block_id': block_id, 'reverse': reverse}



from extras.plugins import PluginMenuButton, PluginMenuItem
from utilities.choices import ButtonColorChoices

from . import config

#
# Sidebar nav content
#

menu_items = (
    PluginMenuItem(
        link=f'plugins:{config.name}:ldp_list',
        link_text='LDP',
        buttons=[
            PluginMenuButton(
                link=f'plugins:{config.name}:ldp_add',
                title='Add LDP Participant',
                icon_class='mdi mdi-plus-thick',
                color=ButtonColorChoices.GREEN,
            )
        ],
    ),
    PluginMenuItem(
        link=f'plugins:{config.name}:bgp_ce_list',
        link_text='BGP CE',
        buttons=[
            PluginMenuButton(
                link=f'plugins:{config.name}:bgp_ce_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                color=ButtonColorChoices.GREEN,
            )
        ],
    ),
    PluginMenuItem(
        link=f'plugins:{config.name}:bgp_pe_list',
        link_text='BGP PE',
        buttons=[
            PluginMenuButton(
                link=f'plugins:{config.name}:bgp_pe_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                color=ButtonColorChoices.GREEN,
            )
        ],
    ),
    PluginMenuItem(
        link=f'plugins:{config.name}:bgp_mesh_list',
        link_text='BGP Mesh',
        buttons=[
            PluginMenuButton(
                link=f'plugins:{config.name}:bgp_mesh_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                color=ButtonColorChoices.GREEN,
            )
        ],
    ),
    PluginMenuItem(
        link=f'plugins:{config.name}:address_family_list',
        link_text='Address Family',
        buttons=[
            PluginMenuButton(
                link=f'plugins:{config.name}:address_family_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                color=ButtonColorChoices.GREEN,
            )
        ],
    ),
    PluginMenuItem(
        link=f'plugins:{config.name}:mpls_instance_list',
        link_text='MPLS Instance',
        buttons=[
            PluginMenuButton(
                link=f'plugins:{config.name}:mpls_instance_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                color=ButtonColorChoices.GREEN,
            )
        ],
    ),
)



from django.core.validators import MinValueValidator, MaxValueValidator
from django.db import models
from django.urls import reverse

from netbox.models import NetBoxModel
from utilities.choices import ChoiceSet

from . import config


class Community_Choices(ChoiceSet):
    key = 'Address_Family.action'

    CHOICES = [
        ('standard', 'Standard'),
        ('extended', 'Extended'),
        ('both', 'Both'),
    ]


class MPLS_Choices(ChoiceSet):
    key = 'MPLS_Instance.action'

    CHOICES = [
        ('mpls_l3_vpn', 'MPLS L3 VPN', 'blue'),
    ]


class LDP(NetBoxModel):
    """Single LDP range."""

    device = models.OneToOneField(
        blank=False,
        on_delete=models.deletion.PROTECT,
        to="dcim.device",
    )

    ldp_enabled_interfaces = models.ManyToManyField(
        to="dcim.interface",
        limit_choices_to=models.Q(device_id=models.F("device")),
        blank=False,
        verbose_name="LDP Interfaces",
    )

    auto_range = models.BooleanField(
        default=True,
        help_text="Whether the lable range should be determined by the device itself.",
    )

    label_start = models.PositiveIntegerField(
        blank=True,
        null=True,
        validators=[MinValueValidator(0), MaxValueValidator(1048575)],
        help_text="Value between [0; 1048575]",
    )

    label_end = models.PositiveIntegerField(
        blank=True,
        null=True,
        validators=[MinValueValidator(0), MaxValueValidator(1048575)],
        help_text="Value between [0; 1048575]",
    )

    comments = models.TextField()

    last_updated = models.DateTimeField(auto_now=True, null=True)

    class Meta:
        verbose_name = 'LDP Participant'
        verbose_name_plural = 'LDP Participants'

    def __str__(self) -> str:
        return f'{self.device}'

    @property
    def range(self):
        return "Auto" if self.auto_range else f"[{self.label_start}; {self.label_end}]"

    def get_absolute_url(self):
        return reverse(f"plugins:{config.name}:ldp", args=[self.pk])


class BGP_CE(NetBoxModel):
    """Single CE Device."""

    name = models.CharField(
        blank=False,
        max_length=50,
    )

    ip_addresses = models.ManyToManyField(
        to="ipam.ipaddress",
        blank=False,
        verbose_name="PE facing IP addresses",
    )

    asn = models.ForeignKey(
        to="ipam.asn",
        blank=False,
        verbose_name="ASN of CE device",
        on_delete=models.deletion.PROTECT,
    )

    comments = models.TextField()

    last_updated = models.DateTimeField(auto_now=True, null=True)

    class Meta:
        verbose_name = 'BGP Customer Edge'
        verbose_name_plural = 'BGP Customer Edges'

    def __str__(self) -> str:
        return f'{self.name}'

    def get_absolute_url(self):
        return reverse(f"plugins:{config.name}:bgp_ce", args=[self.pk])


class BGP_PE(NetBoxModel):
    """Single PE device."""

    device = models.OneToOneField(
        blank=False,
        on_delete=models.deletion.PROTECT,
        to="dcim.device",
    )

    update_source_interface = models.OneToOneField(
        to="dcim.interface",
        limit_choices_to=models.Q(device_id=models.F("device")),
        on_delete=models.deletion.PROTECT,
        blank=False,
        verbose_name="Interfaces used for BGP exchange",
    )

    asn = models.ForeignKey(
        to="ipam.asn",
        blank=False,
        verbose_name="ASN",
        on_delete=models.deletion.PROTECT,
    )

    advertised_customers = models.ManyToManyField(
        to=BGP_CE,
        blank=True,
        verbose_name="Advertised CEs",
    )

    last_updated = models.DateTimeField(auto_now=True, null=True)

    comments = models.TextField()

    class Meta:
        verbose_name = 'BGP Provider Edge'
        verbose_name_plural = 'BGP Provider Edges'

    def __str__(self) -> str:
        return f'{self.device}'

    def get_absolute_url(self):
        return reverse(f"plugins:{config.name}:bgp_pe", args=[self.pk])


class Address_Family(NetBoxModel):
    """Address Family"""

    name = models.CharField(
        blank=False,
        max_length=50,
    )

    vpnv4_address_family = models.BooleanField(
        default=True,
        help_text="Whether the vpnv4-address-family should be active or not",
    )

    vpnv4_community = models.CharField(
        max_length=50,
        null=True,
        blank=True,
        choices=Community_Choices,
    )

    vpnv6_address_family = models.BooleanField(
        default=True,
        help_text="Whether the vpnv6-address-family should be active or not",
    )

    vpnv6_community = models.CharField(
        max_length=50,
        null=True,
        blank=True,
        choices=Community_Choices,
    )

    last_updated = models.DateTimeField(auto_now=True, null=True)

    comments = models.TextField()

    class Meta:
        verbose_name = 'Address Family'
        verbose_name_plural = 'Address Families'

    def __str__(self) -> str:
        return f'{self.name}'

    def get_absolute_url(self):
        return reverse(f"plugins:{config.name}:address_family", args=[self.pk])


class BGP_Mesh(NetBoxModel):
    """BGP Mesh Neighbourhood"""

    name = models.CharField(
        blank=False,
        max_length=50,
    )

    bgp_mesh = models.ManyToManyField(
        to=BGP_PE,
        blank=False,
        verbose_name="BGP PEs",
        related_name="bgp_mesh",
    )

    address_family = models.ForeignKey(
        to=Address_Family,
        blank=False,
        on_delete=models.deletion.PROTECT,
        verbose_name="Address Family",
    )

    last_updated = models.DateTimeField(auto_now=True, null=True)

    comments = models.TextField()

    class Meta:
        verbose_name = 'BGP Mesh'
        verbose_name_plural = 'BGP Meshes'

    def __str__(self) -> str:
        return f'{self.name}'

    def get_absolute_url(self):
        return reverse(f"plugins:{config.name}:bgp_mesh", args=[self.pk])


class MPLS_Instance(NetBoxModel):
    """BGP Mesh Neighbourhood"""

    name = models.CharField(
        blank=False,
        max_length=50,
    )

    ibgp = models.OneToOneField(
        to=BGP_Mesh,
        blank=False,
        on_delete=models.deletion.PROTECT,
        verbose_name="iBGP Network",
        related_name="ibgp",
    )

    service = models.CharField(
        max_length=50,
        blank=False,
        null=False,
        choices=MPLS_Choices,
    )

    last_updated = models.DateTimeField(auto_now=True, null=True)

    comments = models.TextField()

    class Meta:
        verbose_name = 'MPLS Instance'
        verbose_name_plural = 'MPLS Instances'

    def __str__(self) -> str:
        return f'{self.name}'

    def get_absolute_url(self):
        return reverse(f"plugins:{config.name}:mpls_instance", args=[self.pk])

    def get_service_color(self):
        return MPLS_Choices.colors.get(self.service)
